@@ -115,6 +115,7 @@ static VALUE	custom_sym;
 static VALUE	empty_string_sym;
 static VALUE	escape_mode_sym;
 static VALUE	integer_range_sym;
+static VALUE	fast_sym;
 static VALUE	float_prec_sym;
 static VALUE	float_sym;
 static VALUE	huge_sym;
@@ -230,7 +231,7 @@ struct _options	oj_default_options = {
  * - *:mode* [_:object_|_:strict_|_:compat_|_:null_|_:custom_|_:rails_|_:wab_] load and dump modes to use for JSON
  * - *:time_format* [_:unix_|_:unix_zone_|_:xmlschema_|_:ruby_] time format when dumping
  * - *:bigdecimal_as_decimal* [_Boolean_|_nil_] dump BigDecimal as a decimal number or as a String
- * - *:bigdecimal_load* [_:bigdecimal_|_:float_|_:auto_] load decimals as BigDecimal instead of as a Float. :auto pick the most precise for the number of digits.
+ * - *:bigdecimal_load* [_:bigdecimal_|_:float_|_:auto_|_:fast_] load decimals as BigDecimal instead of as a Float. :auto pick the most precise for the number of digits. :float should be the same as ruby. :fast may require rounding but is must faster.
  * - *:create_id* [_String_|_nil_] create id for json compatible object encoding, default is 'json_class'
  * - *:create_additions* [_Boolean_|_nil_] if true allow creation of instances using create_id on load.
  * - *:second_precision* [_Fixnum_|_nil_] number of digits after the decimal when dumping the seconds portion of time
@@ -331,6 +332,7 @@ get_def_opts(VALUE self) {
     switch (oj_default_options.bigdec_load) {
     case BigDec:	rb_hash_aset(opts, bigdecimal_load_sym, bigdecimal_sym);break;
     case FloatDec:	rb_hash_aset(opts, bigdecimal_load_sym, float_sym);	break;
+    case FastDec:	rb_hash_aset(opts, bigdecimal_load_sym, fast_sym);	break;
     case AutoDec:
     default:		rb_hash_aset(opts, bigdecimal_load_sym, auto_sym);	break;
     }
@@ -574,6 +576,8 @@ oj_parse_options(VALUE ropts, Options copts) {
 	    copts->bigdec_load = BigDec;
 	} else if (float_sym == v) {
 	    copts->bigdec_load = FloatDec;
+	} else if (fast_sym == v) {
+	    copts->bigdec_load = FastDec;
 	} else if (auto_sym == v || Qfalse == v) {
 	    copts->bigdec_load = AutoDec;
 	} else {
@@ -803,7 +807,7 @@ oj_parse_opt_match_string(RxClass rc, VALUE ropts) {
 }
 
 /* Document-method: load
- * call-seq: load(json, options) { _|_obj, start, len_|_ }
+ * call-seq: load(json, options={}) { _|_obj, start, len_|_ }
  *
  * Parses a JSON document String into a Object, Hash, Array, String, Fixnum,
  * Float, true, false, or nil according to the default mode or the mode
@@ -887,7 +891,7 @@ load(int argc, VALUE *argv, VALUE self) {
 }
 
 /* Document-method: load_file
- * call-seq: load_file(path, options) { _|_obj, start, len_|_ }
+ * call-seq: load_file(path, options={}) { _|_obj, start, len_|_ }
  *
  * Parses a JSON document String into a Object, Hash, Array, String, Fixnum,
  * Float, true, false, or nil according to the default mode or the mode
@@ -1045,7 +1049,7 @@ safe_load(VALUE self, VALUE doc) {
  */
 
 /* Document-method: dump
- * call-seq: dump(obj, options)
+ * call-seq: dump(obj, options={})
  *
  * Dumps an Object (obj) to a string.
  * - *obj* [_Object_] Object to serialize as an JSON document String
@@ -1143,7 +1147,7 @@ to_json(int argc, VALUE *argv, VALUE self) {
 }
 
 /* Document-method: to_file
- * call-seq: to_file(file_path, obj, options)
+ * call-seq: to_file(file_path, obj, options={})
  *
  * Dumps an Object to the specified file.
  * - *file* [_String_] _path file path to write the JSON document to
@@ -1166,7 +1170,7 @@ to_file(int argc, VALUE *argv, VALUE self) {
 }
 
 /* Document-method: to_stream
- * call-seq: to_stream(io, obj, options)
+ * call-seq: to_stream(io, obj, options={})
  *
  * Dumps an Object to the specified IO stream.
  * - *io* [_IO_] IO stream to write the JSON document to
@@ -1654,6 +1658,7 @@ Init_oj() {
     empty_string_sym = ID2SYM(rb_intern("empty_string"));	rb_gc_register_address(&empty_string_sym);
     escape_mode_sym = ID2SYM(rb_intern("escape_mode"));		rb_gc_register_address(&escape_mode_sym);
     integer_range_sym = ID2SYM(rb_intern("integer_range"));	rb_gc_register_address(&integer_range_sym);
+    fast_sym = ID2SYM(rb_intern("fast"));			rb_gc_register_address(&fast_sym);
     float_prec_sym = ID2SYM(rb_intern("float_precision"));	rb_gc_register_address(&float_prec_sym);
     float_sym = ID2SYM(rb_intern("float"));			rb_gc_register_address(&float_sym);
     huge_sym = ID2SYM(rb_intern("huge"));			rb_gc_register_address(&huge_sym);
