@@ -77,7 +77,9 @@ typedef enum {
 typedef enum {
     BigDec	= 'b',
     FloatDec	= 'f',
-    AutoDec	= 'a'
+    AutoDec	= 'a',
+    FastDec	= 'F',
+    RubyDec	= 'r',
 } BigLoad;
 
 typedef enum {
