@@ -393,8 +393,9 @@ read_num(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 0;
     ni.neg = 0;
-    ni.hasExp = 0;
-    ni.no_big = (FloatDec == pi->options.bigdec_load);
+    ni.has_exp = 0;
+    ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
+    ni.bigdec_load = pi->options.bigdec_load;
 
     if ('-' == *pi->cur) {
 	pi->cur++;
@@ -453,6 +454,7 @@ read_num(ParseInfo pi) {
 	    // except when mimicing the JSON gem or in strict mode.
 	    if (StrictMode == pi->options.mode || CompatMode == pi->options.mode) {
 		int	pos = (int)(pi->cur - ni.str);
+
 		if (1 == pos || (2 == pos && ni.neg)) {
 		    oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "not a number");
 		    return;
@@ -468,18 +470,26 @@ read_num(ParseInfo pi) {
 		if (0 < ni.num || 0 < ni.i) {
 		    dec_cnt++;
 		}
-		ni.num = ni.num * 10 + d;
-		ni.div *= 10;
-		ni.di++;
-		if (INT64_MAX <= ni.div || DEC_MAX < dec_cnt) {
-		    ni.big = 1;
+		if (INT64_MAX <= ni.div) {
+		    if (!ni.no_big) {
+			ni.big = true;
+		    }
+		} else {
+		    ni.num = ni.num * 10 + d;
+		    ni.div *= 10;
+		    ni.di++;
+		    if (INT64_MAX <= ni.div || DEC_MAX < dec_cnt) {
+			if (!ni.no_big) {
+			    ni.big = true;
+			}
+		    }
 		}
 	    }
 	}
 	if ('e' == *pi->cur || 'E' == *pi->cur) {
 	    int	eneg = 0;
 
-	    ni.hasExp = 1;
+	    ni.has_exp = 1;
 	    pi->cur++;
 	    if ('-' == *pi->cur) {
 		pi->cur++;
@@ -490,7 +500,7 @@ read_num(ParseInfo pi) {
 	    for (; '0' <= *pi->cur && *pi->cur <= '9'; pi->cur++) {
 		ni.exp = ni.exp * 10 + (*pi->cur - '0');
 		if (EXP_MAX <= ni.exp) {
-		    ni.big = 1;
+		    ni.big = true;
 		}
 	    }
 	    if (eneg) {
@@ -750,6 +760,59 @@ parse_big_decimal(VALUE str) {
     return rb_funcall(rb_cObject, oj_bigdecimal_id, 1, str);
 }
 
+static long double	exp_plus[] = {
+    1.0,
+    1.0e1,
+    1.0e2,
+    1.0e3,
+    1.0e4,
+    1.0e5,
+    1.0e6,
+    1.0e7,
+    1.0e8,
+    1.0e9,
+    1.0e10,
+    1.0e11,
+    1.0e12,
+    1.0e13,
+    1.0e14,
+    1.0e15,
+    1.0e16,
+    1.0e17,
+    1.0e18,
+    1.0e19,
+    1.0e20,
+    1.0e21,
+    1.0e22,
+    1.0e23,
+    1.0e24,
+    1.0e25,
+    1.0e26,
+    1.0e27,
+    1.0e28,
+    1.0e29,
+    1.0e30,
+    1.0e31,
+    1.0e32,
+    1.0e33,
+    1.0e34,
+    1.0e35,
+    1.0e36,
+    1.0e37,
+    1.0e38,
+    1.0e39,
+    1.0e40,
+    1.0e41,
+    1.0e42,
+    1.0e43,
+    1.0e44,
+    1.0e45,
+    1.0e46,
+    1.0e47,
+    1.0e48,
+    1.0e49,
+};
+
 VALUE
 oj_num_as_value(NumInfo ni) {
     volatile VALUE	rnum = Qnil;
@@ -762,7 +825,7 @@ oj_num_as_value(NumInfo ni) {
 	}
     } else if (ni->nan) {
 	rnum = rb_float_new(0.0/0.0);
-    } else if (1 == ni->div && 0 == ni->exp) { // fixnum
+    } else if (1 == ni->div && 0 == ni->exp && !ni->has_exp) { // fixnum
 	if (ni->big) {
 	    if (256 > ni->len) {
 		char	buf[256];
@@ -793,33 +856,39 @@ oj_num_as_value(NumInfo ni) {
 	    if (ni->no_big) {
 		rnum = rb_funcall(rnum, rb_intern("to_f"), 0);
 	    }
-	} else {
-	    // All these machinations are to get rounding to work better.
-	    long double	d = (long double)ni->i * (long double)ni->div + (long double)ni->num;
+	} else if (FastDec == ni->bigdec_load) {
+	    long double	ld = (long double)ni->i * (long double)ni->div + (long double)ni->num;
 	    int		x = (int)((int64_t)ni->exp - ni->di);
 
-	    // Rounding sometimes cuts off the last digit even if there are only
-	    // 15 digits. This attempts to fix those few cases where this
-	    // occurs.
-	    if ((long double)INT64_MAX > d && (int64_t)d != (ni->i * ni->div + ni->num)) {
-		volatile VALUE	bd = rb_str_new(ni->str, ni->len);
-
-		rnum = rb_rescue2(parse_big_decimal, bd, rescue_big_decimal, bd, rb_eException, 0);
-		if (ni->no_big) {
-		    rnum = rb_funcall(rnum, rb_intern("to_f"), 0);
+	    if (0 < x) {
+		if (x < (int)(sizeof(exp_plus) / sizeof(*exp_plus))) {
+		    ld *= exp_plus[x];
+		} else {
+		    ld *= powl(10.0, x);
 		}
-	    } else {
-		d = roundl(d);
-		if (0 < x) {
-		    d *= powl(10.0L, x);
-		} else if (0 > x) {
-		    d /= powl(10.0L, -x);
+	    } else if (x < 0) {
+		if (-x < (int)(sizeof(exp_plus) / sizeof(*exp_plus))) {
+		    ld /= exp_plus[-x];
+		} else {
+		    ld /= powl(10.0, -x);
 		}
-		if (ni->neg) {
-		    d = -d;
-		}
-		rnum = rb_float_new((double)d);
 	    }
+	    if (ni->neg) {
+		ld = -ld;
+	    }
+	    rnum = rb_float_new((double)ld);
+	} else if (RubyDec == ni->bigdec_load) {
+	    volatile VALUE	sv = rb_str_new(ni->str, ni->len);
+
+	    rnum = rb_funcall(sv, rb_intern("to_f"), 0);
+	} else {
+	    char	*end;
+	    double	d = strtod(ni->str, &end);
+
+	    if ((long)ni->len != (long)(end - ni->str)) {
+		rb_raise(oj_parse_error_class, "Invalid float");
+	    }
+	    rnum = rb_float_new(d);
 	}
     }
     return rnum;
@@ -841,6 +910,12 @@ oj_set_error_at(ParseInfo pi, VALUE err_clas, const char* file, int line, const
     if (p + 3 < end) {
 	*p++ = ' ';
 	*p++ = '(';
+	*p++ = 'a';
+	*p++ = 'f';
+	*p++ = 't';
+	*p++ = 'e';
+	*p++ = 'r';
+	*p++ = ' ';
 	start = p;
 	for (vp = pi->stack.head; vp < pi->stack.tail; vp++) {
 	    if (end <= p + 1 + vp->klen) {
