@@ -31,8 +31,9 @@ typedef struct _numInfo {
     int		infinity;
     int		nan;
     int		neg;
-    int		hasExp;
+    int		has_exp;
     int		no_big;
+    int		bigdec_load;
 } *NumInfo;
 
 typedef struct _parseInfo {
