@@ -407,8 +407,10 @@ read_num(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 0;
     ni.neg = 0;
-    ni.hasExp = 0;
-    ni.no_big = (FloatDec == pi->options.bigdec_load);
+    ni.has_exp = 0;
+    ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
+    ni.bigdec_load = pi->options.bigdec_load;
+
     c = reader_get(&pi->rd);
     if ('-' == c) {
 	c = reader_get(&pi->rd);
@@ -469,18 +471,26 @@ read_num(ParseInfo pi) {
 		if (0 < ni.num || 0 < ni.i) {
 		    dec_cnt++;
 		}
-		ni.num = ni.num * 10 + d;
-		ni.div *= 10;
-		ni.di++;
-		if (INT64_MAX <= ni.div || DEC_MAX < dec_cnt) {
-		    ni.big = 1;
+		if (INT64_MAX <= ni.div) {
+		    if (!ni.no_big) {
+			ni.big = true;
+		    }
+		} else {
+		    ni.num = ni.num * 10 + d;
+		    ni.div *= 10;
+		    ni.di++;
+		    if (INT64_MAX <= ni.div || DEC_MAX < dec_cnt) {
+			if (!ni.no_big) {
+			    ni.big = true;
+			}
+		    }
 		}
 	    }
 	}
 	if ('e' == c || 'E' == c) {
 	    int	eneg = 0;
 
-	    ni.hasExp = 1;
+	    ni.has_exp = 1;
 	    c = reader_get(&pi->rd);
 	    if ('-' == c) {
 		c = reader_get(&pi->rd);
@@ -539,7 +549,8 @@ read_nan(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 1;
     ni.neg = 0;
-    ni.no_big = (FloatDec == pi->options.bigdec_load);
+    ni.no_big = (FloatDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load);
+    ni.bigdec_load = pi->options.bigdec_load;
 
     if ('a' != reader_get(&pi->rd) ||
 	('N' != (c = reader_get(&pi->rd)) && 'n' != c)) {
@@ -736,7 +747,8 @@ oj_sparse2(ParseInfo pi) {
 		ni.infinity = 0;
 		ni.nan = 1;
 		ni.neg = 0;
-		ni.no_big = (FloatDec == pi->options.bigdec_load);
+		ni.no_big = (FloatDec == pi->options.bigdec_load || RubyDec == pi->options.bigdec_load || FastDec == pi->options.bigdec_load);
+		ni.bigdec_load = pi->options.bigdec_load;
 		add_num_value(pi, &ni);
 	    } else {
 		oj_set_error_at(pi, oj_parse_error_class, __FILE__, __LINE__, "invalid token");
