@@ -118,6 +118,17 @@ class CustomJuice < Minitest::Test
     dump_and_load(-2.48e100 * 1.0e10, false)
   end
 
+  def test_float_parse
+    f = Oj.load("12.123456789012345678", mode: :custom, bigdecimal_load: :float);
+    assert_equal(Float, f.class)
+  end
+
+  def test_float_parse_fast
+    f = Oj.load("12.123456789012345678", mode: :custom, bigdecimal_load: :fast);
+    assert_equal(Float, f.class)
+    assert_equal('12.12345678901235', "%0.14f" % [f]) # only care about 16 digits
+  end
+
   def test_nan_dump
     assert_equal('null', Oj.dump(0/0.0, :nan => :null))
     assert_equal('3.3e14159265358979323846', Oj.dump(0/0.0, :nan => :huge))
